<?php

/**
 * @file
 * Contains \Drupal\colorbox_field_formatter\Plugin\Field\FieldFormatter\ColorboxFieldFormatter.
 */

namespace Drupal\colorbox_field_formatter\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Annotation\Translation;
use Drupal\Core\Field\Annotation\FieldFormatter;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Plugin implementation of the 'colorbox_field_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "colorbox_field_formatter",
 *   label = @Translation("Colorbox FF"),
 *   field_types = {
 *     "computed",
 *     "string"
 *   }
 * )
 */
class ColorboxFieldFormatter extends FormatterBase {
  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $config = \Drupal::config('colorbox.settings');
    return array(
      'style' => $config->get('custom.style'),
      'link_type' => 'content',
      'link' => '',
      'width' => '500',
      'height' => '500',
      'iframe' => 0,
      'anchor' => '',
      'class' => '',
    ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form += parent::settingsForm($form, $form_state);

    $form['style'] = array(
      '#title' => $this->t('Style of colorbox'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('style'),
      '#options' => $this->getStyles(),
    );

    $form['link_type'] = array(
      '#title' => $this->t('Link colorbox to'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('link_type'),
      '#options' => $this->getLinkTypes(),
      '#attributes' => array(
        'class' => array('colorbox-field-formatter-link-type'),
      ),
    );
    $form['link'] = array(
      '#title' => $this->t('URI'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('link'),
      '#states' => array(
        'visible' => array(
          'select.colorbox-field-formatter-link-type' => array('value' => 'manual'),
        ),
      ),
    );
    if (\Drupal::moduleHandler()->moduleExists('token')) {
      $form['token_help_wrapper'] = array(
        '#type' => 'container',
        '#states' => array(
          'visible' => array(
            'select.colorbox-field-formatter-link-type' => array('value' => 'manual'),
          ),
        ),
      );
      $form['token_help_wrapper']['token_help'] = array(
        '#theme' => 'token_tree',
        '#token_types' => array('entity' => $form['#entity_type']),
        '#global_types' => FALSE,
      );
    }

    $form['width'] = array(
      '#title' => $this->t('Width'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('width'),
    );
    $form['height'] = array(
      '#title' => $this->t('Height'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('height'),
    );
    $form['iframe'] = array(
      '#title' => $this->t('iFrame Mode'),
      '#type' => 'checkbox',
      '#default_value' => $this->getSetting('iframe'),
    );
    $form['anchor'] = array(
      '#title' => $this->t('Anchor'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('anchor'),
    );
    $form['class'] = array(
      '#title' => $this->t('Class'),
      '#type' => 'textfield',
      '#default_value' => $this->getSetting('class'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = array();

    $styles = $this->getStyles();
    $summary[] = $this->t('Style: @style', ['@style' => $styles[$this->getSetting('style')],]);

    $types = $this->getLinkTypes();
    switch ($this->getSetting('link_type')) {
      case 'manual':
        $summary[] = $this->t('Link to @link', ['@link' => $this->getSetting('link'),]);
        break;

      default:
        $summary[] = $this->t('Link to @link', ['@link' => $types[$this->getSetting('link_type')],]);
        break;
    }

    $summary[] = $this->t('Width: @width', ['@width' => $this->getSetting('width'),]);
    $summary[] = $this->t('Height: @height', ['@height' => $this->getSetting('height'),]);
    $summary[] = $this->t('iFrame Mode: @mode', ['@mode' => ($this->getSetting('iframe') ? $this->t('Yes') : $this->t('No')),]);
    if (!empty($this->getSetting('anchor'))) {
      $summary[] = $this->t('Anchor: #@anchor', ['@anchor' => $this->getSetting('anchor'),]);
    }
    if (!empty($this->getSetting('class'))) {
      $summary[] = $this->t('Classes: @class', ['@class' => $this->getSetting('class'),]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = array();

    foreach ($items as $delta => $item) {
      $output = $this->viewValue($item);
      $url = $this->getUrl($item);
      $options = array(
        'html' => TRUE,
        'attributes' => array(
          'class' => array('colorbox', $this->getSetting('style')),
        ),
        'query' => array(
          'width' => $this->getSetting('width'),
          'height' => $this->getSetting('height'),
        ),
      );
      if ($this->getSetting('iframe')) {
        $options['query']['iframe'] = 'true';
      }
      if (!empty($this->getSetting('anchor'))) {
        $options['query']['fragment'] = $this->getSetting('anchor');
      }
      if (!empty($this->getSetting('class'))) {
        $options['attributes']['class'] = array_merge($options['attributes']['class'], explode(' ', $this->getSetting('class')));
      }

      $url->setOptions($options);
      $ext = $url->isExternal();
      $link = Link::fromTextAndUrl($output, $url);
      $element[$delta] = $link->toRenderable();
    }

    return $element;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {
    // The text value has no text format assigned to it, so the user input
    // should equal the output, including newlines.
    return nl2br(Html::escape($item->value));
  }

  /**
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return \Drupal\Core\Url
   */
  protected function getUrl(FieldItemInterface $item) {
    $entity = $item->getEntity();
    if ($this->getSetting('link_type') == 'content') {
      return $entity->toUrl();
    }

    $link = $this->getSetting('link');
    if (\Drupal::moduleHandler()->moduleExists('token')) {
      $token_service = \Drupal::token();
      $link = $token_service->replace($this->getSetting('link'), array($entity->bundle() => $entity), array('clear' => TRUE));
    }
    return Url::fromUri($link);
  }

  /**
   * Callback to provide an array for a select field containing all supported
   * colorbox styles.
   *
   * @return array
   */
  private function getStyles() {
    $styles = array(
      'default' => $this->t('Default'),
    );
    // TODO: Add more styles to the array
    $config = \Drupal::config('colorbox.settings');

    if (\Drupal::moduleHandler()->moduleExists('colorbox_node')) {
      $styles['colorbox-node'] = $this->t('Colorbox node');
    }

    return $styles;
  }

  /**
   * Callback to provide an arry for a select field containing all link types.
   *
   * @return array
   */
  private function getLinkTypes() {
    return array(
      'content' => $this->t('Content'),
      'manual' => $this->t('Manually provide a link'),
    );
  }

}
